/*  File: analyser-jets.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 16 September 2016
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   local headers  */
#include "analyzer-jets.h"

void UserJets::initfunc()
{
  /*   Create objects to calculate the total cross sections   */
  Hist0D::phys(1,"Total cross section");
  Hist0D::phys(2,"Total cross section (hard)");
  
  /*   Create histograms to calculate the one jet cross sections   */
  unsigned int nbins = 48;
  double pTmin = 250.0_GeV, pTmax = 5050.0_GeV;
  auto bins = spacing<hist1d>::linear(nbins, pTmin, pTmax);
  
  Hist1D::phys(1, "One jet inclusive cross section, full threshold effects", bins);
  Hist1D::phys(2, "One jet inclusive cross section, no Delta", bins);
  Hist1D::phys(3, "One jet inclusive cross section, ThresholdMain", bins);
  Hist1D::phys(4, "One jet inclusive cross section, Std., no threshold effects", bins);
  Hist1D::phys(5, "One jet inclusive cross section at hard interaction", bins);
  
  cout << "UserJets is using jet algorithm " << jetDef.jet_algorithm()
       << " with R = " << jetDef.R() << endl;
}

void UserJets::userfunc(const shower_history<hhc>& history)
{
  constexpr double tonb = 389379.338;
  bool newHard = true;

  /*   the shower stage after the shower evolution   */
  for(auto it = history.cbegin(); it != history.cend(); ++it)
  {
    /*   showered and hard events   */
    auto& p = it->state;
    auto& hard = it.hard();
    auto& phard = hard.state;
    
    // Weights after showering
    auto& weights = it->weights;
    // The weight for a probability conserving shower
    // including factor to use NLO pdfs at hard scale.
    double wStd  = weights.unitary.main;
           wStd *= weights.unitary.pdf_ratio*color_weight(*it)*tonb;
    // The main threshold factor, omitting P^{reg} term and the Delta term.
    double wThresholdMain = wStd*weights.threshold.main;
    // Now include the P^{reg} term.
    double wNoDelta = wThresholdMain*weights.threshold.reg;
    // Now include the Delta term also, giving the full result.
    double wFull = wNoDelta*weights.threshold.delta;
    
    // Weights before showering.
    // If there are multiple branches from the same hard start,
    // we count the hard contribution only once.
    auto hweights = hard.weights;
    double hweight = 0.0;
    if (newHard)
    {
      hweight  = hweights.unitary.main*color_weight(hard)*tonb;
      hweight *= hweights.unitary.pdf_ratio;
    }
    newHard = false;
    
    /*   Total cross sections   */
    Hist0D::physfill(1, wFull);
    Hist0D::physfill(2, hweight);
    
    //   One jet inclusive cross section after showering
    //  -------------------------------------------------
    
    /*   Converting deductor event to FastJet format   */
    make_pseudojets(theparticles, p);
    
    /*    Run fastjet algorithm   */
    fj::ClusterSequence clustSeq(theparticles, jetDef);
    
    /*    Extract inclusive jets sorted by pT (above pTjetmin)   */
    double pTjetmin = 250.0_GeV;
    incJets = clustSeq.inclusive_jets(pTjetmin);
    
    /*    Fill histogram, only count jets that have |y| < 2.0   */
    for(unsigned j = 0; j < incJets.size(); j++)
      if(abs(incJets[j].rap()) < 2.0)
      {
        Hist1D::physfill(1, wFull,          dirac{}, incJets[j].perp());
        Hist1D::physfill(2, wNoDelta,       dirac{}, incJets[j].perp());
        Hist1D::physfill(3, wThresholdMain, dirac{}, incJets[j].perp());
        Hist1D::physfill(4, wStd,           dirac{}, incJets[j].perp());
      }
    
    //   One jet inclusive cross section before showering
    //  --------------------------------------------------
    
    /*   Fill histogram, only count jets that have |y| < 2.0
     *   The two jets are back-to-back, since we have only two
     *   partons in the final state.
     */
    
    if(abs(phard[1].momentum.rapidity()) < 2.0)
    {
      Hist1D::physfill(5, hweight, dirac{}, phard[1].momentum.perp());
    }
    if(abs(phard[2].momentum.rapidity()) < 2.0)
    {
      Hist1D::physfill(5, hweight, dirac{}, phard[2].momentum.perp());
    }
  }
}